package com.chelvc.framework.oauth.context;

import java.io.Serializable;
import java.time.Duration;
import java.util.Map;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.model.Terminal;
import com.chelvc.framework.base.util.ObjectUtils;
import com.chelvc.framework.base.util.StringUtils;
import com.chelvc.framework.redis.util.RedisUtils;
import com.google.common.collect.Maps;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.jwt.Jwt;

/**
 * Oauth上下文工具类
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
public class OauthContextHolder {
    /**
     * 主体身份标识
     */
    public static final String JWT_CLAIM_ID = "id";

    /**
     * JWT身份标识
     */
    public static final String JWT_CLAIM_JTI = "jti";

    /**
     * JWT手机号标识
     */
    public static final String JWT_CLAIM_MOBILE = "mobile";

    /**
     * 业务类型标识
     */
    public static final String JWT_CLAIM_BUSINESS = "business";

    /**
     * 用户账号标识
     */
    public static final String JWT_CLAIM_USER_NAME = "user_name";

    /**
     * 客户端账号标识
     */
    public static final String JWT_CLAIM_CLIENT_ID = "client_id";

    /**
     * 用户权限标识
     */
    public static final String JWT_CLAIM_AUTHORITIES = "authorities";

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(Jwt jwt) {
        return ObjectUtils.ifNull(jwt, t -> ObjectUtils.object2long(t.getClaim(JWT_CLAIM_ID)));
    }

    /**
     * 获取主体身份
     *
     * @param token OAuth2访问令牌对象
     * @return 主体身份标识
     */
    public static Long getId(OAuth2AccessToken token) {
        return ObjectUtils.ifNull(
                ObjectUtils.ifNull(token, OAuth2AccessToken::getAdditionalInformation),
                info -> ObjectUtils.object2long(info.get(JWT_CLAIM_ID))
        );
    }

    /**
     * 获取JWT身份标识
     *
     * @param jwt JWT对象
     * @return JWT身份标识
     */
    public static String getJti(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_JTI), (String) null);
    }

    /**
     * 获取用户标识
     *
     * @param jwt JWT对象
     * @return 用户标识
     */
    public static String getUser(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_USER_NAME), (String) null);
    }

    /**
     * 获取手机号
     *
     * @param jwt JWT对象
     * @return 手机号
     */
    public static String getMobile(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_MOBILE), (String) null);
    }

    /**
     * 获取手机号
     *
     * @param token OAuth2访问令牌对象
     * @return 手机号
     */
    public static String getMobile(OAuth2AccessToken token) {
        return ObjectUtils.ifNull(
                ObjectUtils.ifNull(token, OAuth2AccessToken::getAdditionalInformation),
                info -> StringUtils.ifEmpty((String) info.get(JWT_CLAIM_MOBILE), (String) null)
        );
    }

    /**
     * 获取业务类型
     *
     * @param jwt JWT对象
     * @return 业务类型
     */
    public static String getBusiness(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_BUSINESS), (String) null);
    }

    /**
     * 获取业务类型
     *
     * @param id            会话ID
     * @param redisTemplate Redis操作模版实例
     * @return 业务类型
     */
    public static String getBusiness(@NonNull Serializable id, @NonNull RedisTemplate<String, Object> redisTemplate) {
        return (String) redisTemplate.opsForHash().get(buildAccessTokenKey(id), "business");
    }

    /**
     * 获取业务类型
     *
     * @param token OAuth2访问令牌对象
     * @return 业务类型
     */
    public static String getBusiness(OAuth2AccessToken token) {
        return ObjectUtils.ifNull(
                ObjectUtils.ifNull(token, OAuth2AccessToken::getAdditionalInformation),
                info -> StringUtils.ifEmpty((String) info.get(JWT_CLAIM_BUSINESS), (String) null)
        );
    }

    /**
     * 获取客户端标识
     *
     * @param jwt JWT对象
     * @return 客户端标识
     */
    public static String getClient(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_CLIENT_ID), (String) null);
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息
     */
    public static String getAuthority(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(JWT_CLAIM_AUTHORITIES), (String) null);
    }

    /**
     * 构建令牌标识
     *
     * @param id 会话ID
     * @return 令牌标识
     */
    public static String buildAccessTokenKey(@NonNull Serializable id) {
        return buildAccessTokenKey(id, SessionContextHolder.getTerminal());
    }

    /**
     * 构建令牌标识
     *
     * @param id       会话ID
     * @param terminal 终端信息
     * @return 令牌标识
     */
    public static String buildAccessTokenKey(@NonNull Serializable id, @NonNull Terminal terminal) {
        return String.format("token:%s:%s", id, terminal);
    }

    /**
     * 存储访问令牌
     *
     * @param token         token OAuth2访问令牌对象
     * @param redisTemplate Redis操作模版实例
     */
    public static void storeAccessToken(@NonNull OAuth2AccessToken token,
                                        @NonNull RedisTemplate<String, Object> redisTemplate) {
        String key = buildAccessTokenKey(getId(token));
        if (log.isDebugEnabled()) {
            log.debug("Saving redis token: {}", key);
        }
        Map<String, Object> context = Maps.newHashMapWithExpectedSize(2);
        context.put("value", token.getValue());
        String business = getBusiness(token);
        if (StringUtils.nonEmpty(business)) {
            context.put("business", business);
        }
        long millis = Math.max(token.getExpiration().getTime() - System.currentTimeMillis(), 0);
        RedisUtils.hmset(redisTemplate, key, context, Duration.ofMillis(millis));
    }

    /**
     * 删除访问令牌
     *
     * @param token         token OAuth2访问令牌对象
     * @param redisTemplate Redis操作模版实例
     */
    public static void removeAccessToken(@NonNull OAuth2AccessToken token,
                                         @NonNull RedisTemplate<String, Object> redisTemplate) {
        String key = buildAccessTokenKey(getId(token));
        if (log.isDebugEnabled()) {
            log.debug("Saving redis token: {}", key);
        }
        redisTemplate.delete(key);
    }

    /**
     * 获取访问令牌
     *
     * @param jwt           JWT实例
     * @param redisTemplate Redis操作模版实例
     * @return 访问令牌
     */
    public static String getAccessToken(@NonNull Jwt jwt, @NonNull RedisTemplate<String, Object> redisTemplate) {
        return (String) redisTemplate.opsForHash().get(buildAccessTokenKey(getId(jwt)), "value");
    }
}
