package com.chelvc.framework.security.interceptor;

import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.model.Result;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.base.util.StringUtils;
import com.chelvc.framework.security.config.SecurityProperties;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

/**
 * 请求安全拦截器
 *
 * @author Woody
 * @date 2023/8/3
 */
@Slf4j
@Order(10)
@Component
public class RequestSecurityInterceptor implements Filter {
    /**
     * 安全性校验
     *
     * @param request  Http请求对象
     * @param response Http相应对象
     * @return true/false
     * @throws IOException I/O操作异常
     */
    private boolean validate(HttpServletRequest request, HttpServletResponse response) throws IOException {
        // 判断是否需要放行
        SecurityProperties properties = ApplicationContextHolder.getBean(SecurityProperties.class);
        SecurityProperties.Request config = properties.getRequest();
        if (StringUtils.isEmpty(config.getPermit()) && StringUtils.isEmpty(config.getRequire())) {
            return true;
        }
        String uri = HttpUtils.getRequestURI(request);
        if (StringUtils.isPath(config.getPermit(), uri) || (StringUtils.nonEmpty(config.getRequire())
                && !StringUtils.isPath(config.getRequire(), uri))) {
            return true;
        }

        // 判断请求是否合法
        String timestamp;
        long duration = Math.max(config.getDuration(), 1000);
        if (StringUtils.nonEmpty(request.getHeader(SessionContextHolder.HEADER_DEVICE))
                && StringUtils.nonEmpty(request.getHeader(SessionContextHolder.HEADER_PLATFORM))
                && StringUtils.nonEmpty(request.getHeader(SessionContextHolder.HEADER_TERMINAL))
                && StringUtils.nonEmpty(request.getHeader(SessionContextHolder.HEADER_VERSION))
                && StringUtils.nonEmpty(timestamp = request.getHeader(SessionContextHolder.HEADER_TIMESTAMP))
                && StringUtils.isNumber(timestamp)
                && System.currentTimeMillis() - Long.parseLong(timestamp) <= duration) {
            return true;
        }

        // 无效请求
        log.warn(HttpUtils.getExceptionMessage(request, HttpStatus.FORBIDDEN, "Request invalid"));
        if (StringUtils.isPath(config.getIgnore(), uri)) {
            return true;
        }
        response.setStatus(HttpStatus.FORBIDDEN.value());
        HttpUtils.writeUnifiedResponse(response, Result.build(HttpStatus.FORBIDDEN));
        return false;
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        if (this.validate((HttpServletRequest) request, (HttpServletResponse) response)) {
            chain.doFilter(new RepeatableRequestWrapper((HttpServletRequest) request), response);
        }
    }
}
